#include <fstream>
#include <iostream>
#include <new>

#include "MyTmpltLib.h"

using namespace std;

#include "Course.cpp"
#include "Student.cpp"
#include "SIndexEntry.h"

struct ProgXptn
{
   enum { allocFailed, cannotOpen };
   char functionName[100];
   int  errorCode;
   char errStrVal[100];
   ProgXptn( const char* fn, int ec, const char* sv="" )
      :  errorCode( ec )
   {  strncpy( functionName, fn, 99 );  functionName[99] = '\0';
      strncpy( errStrVal, sv, 99 );  errStrVal[99] = '\0';  }
}; // ProgXptn

void loadIndex( string flNm,
                PSIndexEntry& index, unsigned int& ndxSz );
void retrieve( string flNm, PSIndexEntry index, int ndxSz );

int main()
{
   SIndexEntry*  sIndex;
   unsigned int  ndxSz;
   try
   {
      loadIndex( "students.ndx", sIndex, ndxSz );
      retrieve( "students.dta", sIndex, ndxSz );
   } // try
   catch( ProgXptn& x )
   {
      switch ( x.errorCode )
      {
         case ProgXptn::allocFailed:
            cout << "no dynamic memory in " << x.functionName << endl;
            break;
         case ProgXptn::cannotOpen:
            cout << "cannot open file " << x.errStrVal << " in "
                 << x.functionName << endl;
            break;
         default:
            cout << "unexpected ProgXptn from " << x.functionName << endl;
      } // switch
   } // catch( ProgXptn
   catch( MyTmpltLibXptn& x )
   {
      switch ( x.errCode )
      {
         case MyTmpltLibXptn::noArray:
            cout << x.funcName << " called with NULL array pointer"
                 << endl;
            break;
         case MyTmpltLibXptn::paramError:
            cout << x.funcName << ": param error ni(" << x.errIntVal1
                 << ") and nf(" << x.errIntVal2 << ")" << endl;
            break;
         case MyTmpltLibXptn::allocFailed:
            cout << "no dynamic memory in " << x.funcName << endl;
            break;
         case MyTmpltLibXptn::cannotCreate:
            cout << "cannot create file " << x.errStrVal << " in "
                 << x.funcName << endl;
            break;
         case MyTmpltLibXptn::cannotWrite:
            cout << "cannot write to file " << x.errStrVal << " in "
                 << x.funcName << endl;
            break;
         default:
            cout << "unexpected MyTmpltLibXptn from "
                 << x.funcName << endl;
      } // switch
   } // catch( MyTmpltLibXptn
   catch( StudentXptn& x )
   {
      switch ( x.errorCode )
      {
         case StudentXptn::allocFailed:
            cout << "Student " << x.objKey
                 << ": no dynamic memory in " << x.methodName << endl;
            break;
         case StudentXptn::negIdNum:
            cout << "Student " << x.objKey << ": illegal id num ("
                 << x.errIntVal << ") in " << x.methodName << endl;
            break;
         case StudentXptn::incomplete:
            cout << "incomplete data for Student " << x.objKey
                 << " in " << x.methodName << endl;
            break;
         case StudentXptn::fileNotOpen:
            cout << "file " << x.errStrVal << " not open in "
                 << x.methodName << endl;
            break;
         case StudentXptn::cannotWrite:
            cout << "cannot write to file " << x.errStrVal << " in "
                 << x.methodName << endl;
            break;
         case StudentXptn::cannotRead:
            cout << "cannot read from file " << x.errStrVal << " in "
                 << x.methodName << endl;
            break;
         default:
            cout << "unexpected StudentXptn from " << x.methodName << endl;
      } // switch
   } // catch( StudentXptn
   catch( ... )
   {  cout << "unexpected exceptioon" << endl;  }
system("pause");
}

void loadIndex( string flNm,
                PSIndexEntry& sIndex, unsigned int& ndxSz )
{
   ifstream bin( flNm.c_str(), ios_base::binary );
   if ( bin.fail() )
      throw ProgXptn( "loadIndex", ProgXptn::cannotOpen,
                      flNm.c_str() );
   
   bin.seekg( 0, ios_base::end );
   unsigned int flSize( bin.tellg() );
   ndxSz = flSize/sizeof(SIndexEntry);

   try {  sIndex = new SIndexEntry[ndxSz+1];  }
   catch( bad_alloc )
   {  throw ProgXptn( "loadIndex", ProgXptn::allocFailed );  }

   bin.seekg( 0 );
   for ( int k(0); k < ndxSz; ++k )
      bin.read( reinterpret_cast<char*>(&sIndex[k]),
                sizeof(SIndexEntry) );
   bin.close();
} // loadIndex

void retrieve( string flNm, PSIndexEntry sIndex, int ndxSz )
{
   ifstream bin( flNm.c_str(), ios_base::binary );
   if ( bin.fail() )
      throw ProgXptn( "retrieve", ProgXptn::cannotOpen,
                      flNm.c_str() );

   string line;
   cout << "Student Id Number: ";  getline( cin, line, '\n' );
   while ( line != "" )
   {
      int idNum( atoi(line.c_str()) );
      int ndx( linSearch(sIndex, ndxSz, 0, ndxSz-1, SIndexEntry(idNum)) );
      if ( ndx < 0 )
         cout << "unknown Student Id Number" << endl;
      else
      {
         Student oneStudent;
         bin.seekg( sIndex[ndx].loc );
         oneStudent.load( bin );
         oneStudent.display( cout );
      }
      cout << "Student Id Number: ";  getline( cin, line, '\n' );
   } // while
   bin.close();
} // retrieve
